#!/usr/bin/perl -w

# Copyright (c) 2008  Peter Pentchev
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Ringlet: log-reformat.pl 2003 2008-05-26 16:19:11Z roam $

use strict;

use Getopt::Std;
use URI::Escape;

sub debug($);
sub usage($);
sub version();

sub reformat_none($);
sub reformat_vsftpd_escape($);

my ($debug, $fmt) = (0, 'vsftpd_escape');
my %reformat = (
	'none'		=> \&reformat_none,
	'vsftpd_escape'	=> \&reformat_vsftpd_escape,
);

MAIN:
{
	my ($fmtfunc, $line, $mode, $outf, $outfname);
	my (%opts);

	getopts('f:hm:o:Vv', \%opts) or usage(1);
	version() if defined($opts{'V'});
	usage(0) if defined($opts{'h'});
	exit(0) if (defined($opts{'h'}) || defined($opts{'V'}));
	$debug++ if defined($opts{'v'});
	if (defined($opts{'o'}) && $opts{'o'} ne '-') {
		$outfname = $opts{'o'};
	}
	if (defined($opts{'m'})) {
		if (!defined($opts{'o'})) {
			die("The -m option may not be used without -o\n");
		} elsif ($opts{'m'} !~ /[0-7]+/) {
			die("Only octal modes supported so far\n");
		}
		$mode = $opts{'m'};
	}

	$fmt = $opts{'f'} if defined($opts{'f'});
	if ($fmt eq 'list') {
		print map "$_\n", sort keys %reformat;
		exit(0);
	} elsif (!defined($reformat{$fmt})) {
		die("Unknown reformat specification '$fmt', try '-f list'\n");
	}
	
	if (defined($outfname)) {
		open($outf, '>', $outfname) or
		    die("Opening output file '$outfname': $!\n");
		if (defined($opts{'m'})) {
			chmod(oct($mode), $outfname) or
			    die("Setting mode '$mode' on output file ".
			    "$outfname: $!\n");
		}
	} else {
		$outf = \*STDOUT;
	}
	$fmtfunc = $reformat{$fmt};
	while ($line = <>) {
		chomp $line;
		debug("Read a line: $line");
		$line = &{$fmtfunc}($line);
		print $outf "$line\n";
	}
	if (defined($outfname)) {
		close($outf);
	}
}

# Function:
#	debug()		- display diagnostic information if requested
# Inputs:
#	$str		- the string to display
# Returns:
#	Nothing
# Modifies:
#	Nothing by itself; writes to stderr if requested.
#
sub debug($)
{
	print STDERR "$_[0]\n" if $debug;
}

# Function:
#	reformat_none()	- just pass the log lines through
# Inputs:
#	$line		- the line to reformat
# Returns:
#	The input line, unmodified
# Modifies:
#	Nothing
#
sub reformat_none($)
{
	return $_[0];
}

# Function:
#	reformat_vsftpd_escape()	- URL-escape a filename
# Inputs:
#	$line		- the line to reformat
# Returns:
#	The reformatted line
# Modifies:
#	Nothing
#
sub reformat_vsftpd_escape($)
{
	my ($line) = @_;
	my ($before, $filename, $after);

	if ($line !~ /^((\S+\s+){8})(.*)((\s+\S+){9})$/) {
		debug("- not a vsftpd line");
		return $line;
	}
	($before, $filename, $after) = ($1, $3, $4);
	$filename = uri_escape($filename, '^A-Za-z0-9.+/:_-');
	$line = "$before$filename$after";
	debug("- vsftpd: converted to $line");
	return $line;
}

# Function:
#	usage()		- display program usage information
# Inputs:
#	$err		- is this an error or a normal -h request?
# Returns:
#	Nothing, dies if $err is set.
# Modifies:
#	Nothing by itself; writes to stdout or stderr.
#
sub usage($)
{
	my ($err) = @_;
	my $s = "Usage: log-reformat [-hVv] [-f format] [-m mode]\n".
	    "[-o outfile]\n".
	    "\t-f\tspecify the reformatting to do;\n".
	    "\t\t(try '-f list' for a list of the available options)\n".
	    "\t-h\tdisplay this help message and exit;\n".
	    "\t-m\tspecify the access permissions for the output file;\n".
	    "\t-o\tspecify the output file, defaults to standard output;\n".
	    "\t-v\tverbose operation - display diagnostic output;\n".
	    "\t-V\tdisplay version information and exit.\n";

	if ($err) {
		die($s);
	} else {
		print $s;
	}
}

# Function:
#	version()	- display program version information
# Inputs:
#	none
# Returns:
#	Nothing
# Modifies:
#	Nothing by itself; writes to stdout.
#
sub version()
{
	print "log-reformat 0.03\n";
}
